/*
 * ambe_pass.c
 *
 * Implements complex communication function include interrupt-handling and timer
 * based sleep / watchdog.
 *
 *  Created on: 17.01.2010
 *      Author: DO1FJN, Jan Alte
 *
 *
 * This file is part of the DV-modem firmware (DVfirmware).
 * For general infomation about DVfirmware see "dv-main.c".
 *
 * DVfirmware is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * The DVfirmware is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this package. If not, see <http://www.gnu.org/licenses/>.
 *
 *
 * ToDo:
 * -Check Power-Down vs. Hard-Reset Current
 * -Slip-Control
 *
 * Done:
 * +Bug: Kein Einschalten bei Decode nach langer Ruhe!
 * +PreDefined DTMF Tone Packets
 * +easy VAD on/off define
 * +ambe_powerdown()
 * +VoiceBuf System verbessern
 *
 */


#include <math.h>

#include "ambe_func.h"
#include "hw_defs.h"
#include "gpio_func.h"
#include "int_func.h"
#include "intc.h"
#include "compiler.h"
#include <string.h>			// für memcpy, memset

#ifndef AMBEACT


#define AMBE_OPERATEID			0x0000
#define AMBE_PWRDOWNID			0x5500
#define AMBE_ID_MASK			0xFF00
#define AMBE_CTRL1_CNI_OFFSET		1
#define AMBE_CTRL1_LFI_OFFSET		7
#define AMBE_CTRL1_ID_OFFSET		8
#define AMBE_CTRL1_CNI_MASK		(1<<AMBE_CTRL1_CNI_OFFSET)
#define AMBE_CTRL1_LFI_MASK		(1<<AMBE_CTRL1_LFI_OFFSET)

#define AMBE_CTRL2_RIS_OFFSET		0
#define AMBE_CTRL2_EE_OFFSET		2
#define AMBE_CTRL2_SL_OFFSET		3
#define AMBE_CTRL2_VAD_OFFSET		5
#define AMBE_CTRL2_VOL_OFFSET		8

#define AMBE_CTRL2_RIS_BOTH		0x0
#define AMBE_CTRL2_RIS_ENCODER		0x1
#define AMBE_CTRL2_RIS_DECODER		0x2
#define AMBE_CTRL2_RIS_NONE		0x3
#define AMBE_CTRL2_RIS_MASK		0x0003

#define AMBE_CTRL2_EE_MASK		(1<<AMBE_CTRL2_EE_OFFSET)
#define AMBE_CTRL2_SL_MASK		(1<<AMBE_CTRL2_SL_OFFSET)
#define AMBE_CTRL2_VAD_MASK		(1<<AMBE_CTRL2_VAD_OFFSET)

#define AMBE_CTRL2_ENCODE		(AMBE_CTRL2_VAD_MASK|AMBE_CTRL2_EE_MASK)

#define AMBE_TIMER_CLOCK		((MASTERCLOCK+64)/128)

#define AMBE_TIMETOBOOT			134	// Zeit, der der AMBE max. zum Aufwachen benötigt
#define AMBE_FRAMETIMEOUT		24	// bei Überschreiten übernimmt Timer des TX
#define AMBE_TIMETORESET		2	// Zeit nach Powerdown, wenn AMBE erneut starten soll

// Gemessene Bootzeit: 124,05ms bis erstes Paket behandelt


// *** Globale Variablen und Buffer ***

tAMBEinframe	ambe_input;		// To-be-Transmit AMBE-Frame
tAMBEoutframe	ambe_output;		// Last Received AMBE-Frame

unsigned int	ambe_rxframe_count;
unsigned int	ambe_idle_count;
unsigned int	ambe_powerdown_to;
unsigned int	ambe_dtmf_count;

tambestate	AMBEstate = AMBEnoinit;		// Zustand AMBE-Chip
tambestate	prevAMBEstate = AMBEnoinit;	// Zustand brei letzter newstate Abfrage

unsigned short	BitErrorRate;


const char DtmfTone[16][AMBE_USEDDATA] = {
  {0xd3, 0xcb, 0x20, 0xd3, 0x03, 0x1d, 0x34, 0x41, 0x10},	// 1
  {0x83, 0xca, 0x24, 0xa2, 0x45, 0x11, 0x44, 0xc6, 0x2c},	// 4
  {0x83, 0x4b, 0x38, 0xc3, 0x04, 0x0d, 0x45, 0x8e, 0x10},	// 7
  {0xd3, 0xcb, 0x3c, 0xb2, 0x02, 0x05, 0x35, 0x0b, 0x2c},	// *
  {0xd3, 0xcb, 0x20, 0xd3, 0x03, 0x5d, 0x34, 0x41, 0x10},	// 2
  {0x83, 0xca, 0x24, 0xa2, 0x45, 0x51, 0x44, 0xc6, 0x2c},	// 5
  {0x83, 0x4b, 0x38, 0xc3, 0x04, 0x4d, 0x45, 0x8e, 0x10},	// 8
  {0xd3, 0xcb, 0x3c, 0xb2, 0x02, 0x45, 0x35, 0x0b, 0x2c},	// 0
  {0xd3, 0xcb, 0x20, 0xd3, 0x13, 0x1d, 0x34, 0x41, 0x10},	// 3
  {0x83, 0xca, 0x24, 0xa2, 0x55, 0x11, 0x44, 0xc6, 0x2c},	// 6
  {0x83, 0x4b, 0x38, 0xc3, 0x14, 0x0d, 0x45, 0x8e, 0x10},	// 9
  {0xd3, 0xcb, 0x3c, 0xb2, 0x12, 0x05, 0x35, 0x0b, 0x2c},	// #  
  {0xd3, 0xcb, 0x20, 0xd3, 0x13, 0x5d, 0x34, 0x41, 0x10},	// A
  {0x83, 0xca, 0x24, 0xa2, 0x55, 0x51, 0x44, 0xc6, 0x2c},	// B
  {0x83, 0x4b, 0x38, 0xc3, 0x14, 0x4d, 0x45, 0x8e, 0x10},	// C
  {0xd3, 0xcb, 0x3c, 0xb2, 0x12, 0x45, 0x35, 0x0b, 0x2c}	// D
};

const char SilenceFrame[AMBE_USEDDATA] = {
  0x8e, 0x4f, 0xb8, 0xb0, 0xd5, 0x5f, 0x2b, 0xa0, 0xe8
};


char 	VoiceBuf[AMBE_USEDDATA];	// zeitweiliger Sprachdaten-Buffer
char 	*VoicePktPtr;			// Zeiger auf aktuelle Daten

// *** Forward ***

void ambe_start(void);
void ambe_start_transmit(void);


// *** Timer-Routinen ***

__inline void ambe_timer_start(void) {
  AVR32_TC.channel[AMBE_TIMER_CH].sr;
  AVR32_TC.channel[AMBE_TIMER_CH].ier = AVR32_TC_CPCS_MASK;
  AVR32_TC.channel[AMBE_TIMER_CH].ccr = AVR32_TC_SWTRG_MASK | AVR32_TC_CLKEN_MASK;
}

__inline void ambe_timer_stop(void) {  // Stop Watchdog-Timer
  AVR32_TC.channel[AMBE_TIMER_CH].idr = 0xFF;
  AVR32_TC.channel[AMBE_TIMER_CH].ccr = AVR32_TC_CLKDIS_MASK;
  AVR32_TC.channel[AMBE_TIMER_CH].sr;	// Clear Pending INT-Requests
}

__inline void ambe_restart_timer(void) {
  AVR32_TC.channel[AMBE_TIMER_CH].ccr = AVR32_TC_SWTRG_MASK; // | AVR32_TC_CLKEN_MASK;
}

__inline void ambe_timer_setms(unsigned int mscnt) {	// ms-max = 138!!!
  AVR32_TC.channel[AMBE_TIMER_CH].rc = (mscnt*AMBE_TIMER_CLOCK)/1000;
}

#define ambe_timer_resetwd()	ambe_timer_setms(AMBE_FRAMETIMEOUT)



// *** ISR für den Timer (Überwachung) ***

#if __GNUC__
__attribute__((__interrupt__))
#elif __ICCAVR32__
__interrupt
#endif
static void ambe_timer_TurnOff(void) {	// Ende Gelände: AMBE ausschalten
  ambe_stop();		// erledigt per timer_stop auch das INT-ACK (lesen SR)
//  ambe_timer_stop(); Test
}


#if __GNUC__
__attribute__((__interrupt__))
#elif __ICCAVR32__
__interrupt
#endif
static void ambe_timer_TurnOn(void) {	// AMBE wieder einschalten
  ambe_timer_stop();			// TurnOff & acknowledge INT
  ambe_start();
}


#if __GNUC__
__attribute__((__interrupt__))
#elif __ICCAVR32__
__interrupt
#endif
static void ambe_timer_Sleep(void) {	// Abschalt-Timer für wartenden AMBE
  AVR32_TC.channel[AMBE_TIMER_CH].sr;	// Read Status to acknowledge INT
  ambe_idle_count++;
  if (ambe_input.Ctrl1&AMBE_PWRDOWNID) {	// Power-Down gesetzt
    tambestate beforestop = AMBEstate;
    ambe_stop();
    if (beforestop==AMBEbooting) {	// Sonderfall -> sofort wieder einschalten
      INTC_register_interrupt(ambe_timer_TurnOn, AMBE_TIMER_IRQ, AMBE_TIMER_PRIO);
      ambe_timer_setms(AMBE_TIMETORESET);
      ambe_timer_start();
    } // fi restart
  } else if (ambe_idle_count > ambe_powerdown_to) {
    ambe_input.Ctrl1 = AMBE_PWRDOWNID;
    ambe_start_transmit();
    AMBEstate = AMBEpowerdown;
    eic_disableint(AMBE_EPR_INT);	// Keine weiteren AMBE-Pakete verarbeiten
  } // fi
}


#if __GNUC__
__attribute__((__interrupt__))
#elif __ICCAVR32__
__interrupt
#endif
static void ambe_timer_Decode(void) {	//
  AVR32_TC.channel[AMBE_TIMER_CH].sr;	// Read Status to acknowledge INT
  ambe_start_transmit();		// TimeOut, Decode LFI->CNI...
  ambe_timer_setms(20);
  if (ambe_input.Ctrl2&AMBE_CTRL2_SL_MASK) {
    INTC_register_interrupt(ambe_timer_Sleep, AMBE_TIMER_IRQ, AMBE_TIMER_PRIO);
  }
}




// *** ISR für SSC Synchronous Serial Controller Receive ***


#if __GNUC__
__attribute__((__interrupt__))
#elif __ICCAVR32__
__interrupt
#endif
static void ambe_rxfinish_int(void) {
  AVR32_SSC.cr  = AVR32_SSC_CR_RXDIS_MASK;	// Stop RX of SSC
  // Int bleibt aktiv solange tcr==0, daher schon mal init
  AVR32_PDCA.channel[AMBE_CHANNEL0].mar = (U32)&ambe_output.Ctrl1;
  AVR32_PDCA.channel[AMBE_CHANNEL0].tcr = AMBE_FRAMESIZE-1;
  eic_clrline(AMBE_EPR_INT);
  if (AMBEstate > AMBEpowerdown)		// No OFF-Req.
    eic_reenableint(AMBE_EPR_INT);		// Activate EPR-Int (sense next packet rdy)

  ambe_rxframe_count++;
  if ((ambe_input.Ctrl2&AMBE_CTRL2_RIS_MASK)==AMBE_CTRL2_RIS_NONE) {
    // (Rate ist gesetzt - Normalbetrieb)
    ambe_idle_count++;
    if (ambe_idle_count >= ambe_powerdown_to) ambe_standby();

    if (ambe_output.Ctrl2&AMBE_CTRL2_EE_MASK) {	// Arbeitet als Decoder oder DTMF-Geber?

      if (ambe_dtmf_count>0) {			// DTMF statt Decoding
 	ambe_dtmf_count--;
        ambe_restart_timer();			// TimeOut-Timer zurücksetzen
	if ((ambe_input.DtmfCtrl&0xF0)==0x80) {	// one of the 16 defined tones
	  VoicePktPtr = (char *)DtmfTone[ambe_input.DtmfCtrl&0x0F];
	} else {		// Silence-Frame
          VoicePktPtr = (char *)SilenceFrame;
	}
	if (ambe_dtmf_count==0) {
	   LSB(ambe_input.DtmfCtrl) = 0xFF;
	   if (AMBEstate==AMBEencoding)
	     LSB(ambe_input.Ctrl2) = AMBE_CTRL2_ENCODE;
	} // fi Ende
	ambe_start_transmit();
      } else if ( (ambe_input.Ctrl1&AMBE_CTRL1_CNI_MASK) == 0 ) {
	BitErrorRate = ambe_output.BER;
      } // fi fresh Voice

    } else {	// *** ist Encoder *** (Out-EE-Bit ist 1 bei DECODER!!!)
      ambe_restart_timer();			// TimeOut-Timer zurücksetzen
      VoicePktPtr = ambe_output.Data;
      if (ambe_input.Ctrl2&AMBE_CTRL2_SL_MASK) { // Standby erwünscht
	INTC_register_interrupt(ambe_timer_Sleep, AMBE_TIMER_IRQ, AMBE_TIMER_PRIO);
	ambe_timer_setms(20);
        ambe_start_transmit();			// zum AMBE
      } // fi stby
    } // esle

  } else {	// RateInfo!=NONE
    // Vergleiche Rate von zuletzt gesendeten Paket:
    if ( (!memcmp(ambe_input.Rate, ambe_output.Rate, sizeof(ambe_output.Rate))) && \
      ((ambe_input.Ctrl2^ambe_output.Ctrl2)&AMBE_CTRL2_EE_MASK) ) {
      ambe_input.Ctrl2 |= AMBE_CTRL2_RIS_NONE;
      ambe_input.Ctrl2 &= ~AMBE_CTRL2_SL_MASK;
      ambe_input.Ctrl1 |= AMBE_CTRL1_CNI_MASK;
      if (AMBEstate > AMBEpowerdown)		// No OFF-Req.
        AMBEstate = (ambe_output.Ctrl2&AMBE_CTRL2_EE_MASK)?AMBEdecoding:AMBEencoding;
    } // fi Rate/Mode gleich
    if (ambe_input.Ctrl2&AMBE_CTRL2_EE_MASK) {	// Encoding-Mode?
      ambe_start_transmit();			// dann Sende gleich zum AMBE
      INTC_register_interrupt(ambe_timer_TurnOff, AMBE_TIMER_IRQ, AMBE_TIMER_PRIO);
    } else {
      INTC_register_interrupt(ambe_timer_Decode, AMBE_TIMER_IRQ, AMBE_TIMER_PRIO);
    }
    ambe_timer_stop();				// Verhindere Timer-Trigger durch RC-Ä
    ambe_timer_setms(AMBE_FRAMETIMEOUT);	// TimeOut, wenn kein Pkt nach 24ms da ist
    ambe_timer_start();
  } // esle Rate/Mode not set

}



#if __GNUC__
__attribute__((__interrupt__))
#elif __ICCAVR32__
__interrupt
#endif
static void ambe_ssc_int(void) {
  U32 Status = AVR32_SSC.sr;// & AVR32_SSC.imr;	// Read+Reset Status Register
  if (Status&AVR32_SSC_SR_RXRDY_MASK) {		// Received a word?
    ambe_output.Header = AVR32_SSC.rhr;
    if (ambe_output.Header == AMBE_HEADER) {	// wenn Header empfangen...
      AVR32_SSC.idr = AVR32_SSC_RXRDY_MASK;	// ...SSC-RxRdy-Int de- ...
      AVR32_PDCA.channel[AMBE_CHANNEL0].cr = AVR32_PDCA_TEN_MASK; // ...und PDAC aktivieren.
      if (VoicePktPtr==ambe_output.Data) { 	// Sprachdaten-Zeiger auf Buffer
 	VoicePktPtr = VoiceBuf;			// setzen, wenn auf Data zeigt.
      } // fi
      memcpy(VoiceBuf, ambe_output.Data, AMBE_USEDDATA);
    } // fi Frame-Header
  } // fi RXRDY
  if (Status&AVR32_SSC_SR_TXEMPTY_MASK) { // Finish Transmitting?
    AVR32_SSC.idr = AVR32_SSC_TXEMPTY_MASK;	// Turn Off IRQ Event
    AVR32_SSC.cr  = AVR32_SSC_CR_TXDIS_MASK;	// Turn Off Transmitter
  } // fi txempty
}


// *** ISR für Packet-Ready Int ***

#if __GNUC__
__attribute__((__interrupt__))
#elif __ICCAVR32__
__interrupt
#endif
static void ambe_epr_int(void) {
  eic_disableint(AMBE_EPR_INT);
  AVR32_PDCA.channel[AMBE_CHANNEL0].cr  = AVR32_PDCA_ECLR_MASK|AVR32_PDCA_TDIS_MASK;
  // PDAC vorbereiten: 23 weitere Words einlesen...
  AVR32_PDCA.channel[AMBE_CHANNEL0].mar = (U32)&ambe_output.Ctrl1;
  AVR32_PDCA.channel[AMBE_CHANNEL0].tcr = AMBE_FRAMESIZE-1;
  // einzelne Words empfangen, auf AMBE-Header warten:
  AVR32_SSC.ier = AVR32_SSC_RXRDY_MASK;
  AVR32_SSC.cr  = AVR32_SSC_CR_RXEN_MASK;	// start RX-Op of SSC
}



#if __GNUC__
__attribute__((__interrupt__))
#elif __ICCAVR32__
__interrupt
#endif
static void ambe_txfinish_int(void) {
  AVR32_PDCA.channel[AMBE_CHANNEL1].idr = AVR32_PDCA_IER_TRC_MASK;
  AVR32_SSC.ier = AVR32_SSC_TXEMPTY_MASK;
  if ((ambe_input.Ctrl2&AMBE_CTRL2_EE_MASK)==0) {	// Decoding-Mode?
    switch (ambe_input.Ctrl1&(AMBE_CTRL1_LFI_MASK|AMBE_CTRL1_CNI_MASK)) {
      case 0:	// No LFI & CNI (actual, valid voice data)?
	LSB(ambe_input.Ctrl1) = AMBE_CTRL1_LFI_MASK; // LFI on
	break;
      case AMBE_CTRL1_LFI_MASK:	// LFI off, CNI on
	LSB(ambe_input.Ctrl1) = AMBE_CTRL1_CNI_MASK; // CNI on
	break;
      } // hctiws
  } // fi encoding
}



void ambe_start_transmit(void) {
  if ((AVR32_SSC.sr&AVR32_SSC_SR_TXEN_MASK)==0) {
    AVR32_PDCA.channel[AMBE_CHANNEL1].cr  = AVR32_PDCA_ECLR_MASK|AVR32_PDCA_TDIS_MASK;
    AVR32_PDCA.channel[AMBE_CHANNEL1].mar = (U32)&ambe_input;
    AVR32_PDCA.channel[AMBE_CHANNEL1].tcr = AMBE_FRAMESIZE;
    AVR32_PDCA.channel[AMBE_CHANNEL1].ier = AVR32_PDCA_IER_TRC_MASK;
    AVR32_SSC.cr  = AVR32_SSC_CR_TXEN_MASK;	// SSC: Transmit Enable
    AVR32_SSC.sr;
    AVR32_PDCA.channel[AMBE_CHANNEL1].cr  = AVR32_PDCA_TEN_MASK; // PDAC-TX aktiv
  }
}


void ambe_start(void) {
  tambestate startstate = AMBEstate;
  if ((startstate==AMBEnoinit)||(startstate > AMBEbooting))
    return;
  AMBEstate = AMBEbooting;
  if (startstate==AMBEoff) {
    ambe_dtmf_count    = 0;
    ambe_idle_count    = 0;
    ambe_rxframe_count = 0;
    cpld_config(CPLDcfg_ambepas);		// Starts AMBE2020 from Reset
    ambe_timer_setms(AMBE_TIMETOBOOT);		// gib AMBE 135ms zum Booten (124.05ms nötig)
    INTC_register_interrupt(ambe_timer_TurnOff, AMBE_TIMER_IRQ, AMBE_TIMER_PRIO);
    ambe_timer_start();
    cpld_start();				// erst jetzt SSC benutzbar
    eic_clrline(AMBE_EPR_INT);
    eic_reenableint(AMBE_EPR_INT);		// Activate EPR-Int (sense next packet rdy)
    AVR32_SSC.sr;
    AVR32_PDCA.channel[AMBE_CHANNEL0].ier = AVR32_PDCA_IER_TRC_MASK;
  } // fi
}



/*! \name AMBE API-Functions
 */
//! @{


void ambe_stop(void) {
  eic_disableint(AMBE_EPR_INT);
  AVR32_SSC.idr = 0xFFFF;		// SSC Ints off
  AVR32_PDCA.channel[AMBE_CHANNEL0].idr = AVR32_PDCA_IDR_TERR_MASK|AVR32_PDCA_IDR_TRC_MASK|AVR32_PDCA_IDR_RCZ_MASK;
  AVR32_PDCA.channel[AMBE_CHANNEL1].idr = AVR32_PDCA_IDR_TERR_MASK|AVR32_PDCA_IDR_TRC_MASK|AVR32_PDCA_IDR_RCZ_MASK;
  AVR32_PDCA.channel[AMBE_CHANNEL0].cr  = AVR32_PDCA_TDIS_MASK;
  AVR32_PDCA.channel[AMBE_CHANNEL1].cr  = AVR32_PDCA_TDIS_MASK;
  AVR32_SSC.cr = 0x0202;		// Stop Both
  ambe_timer_stop();
  cpld_config(CPLDdisabled);
  ambe_rxframe_count = 0;
  ambe_dtmf_count    = 0;
  ambe_idle_count    = 0;
  AMBEstate = AMBEoff;
}


void ambe_standby(void) {
  if (AMBEstate > AMBEpowerdown) {
    ambe_input.Ctrl2 |= AMBE_CTRL2_SL_MASK;	// enable Frame-Based Sleep Mode
    AMBEstate = AMBEsleep;
    ambe_idle_count = 0;
  } // fi acticw
}


void ambe_encode(void) {
  if ((AMBEstate != AMBEencoding) && (AMBEstate != AMBEnoinit)) {
    ambe_input.Ctrl1 = AMBE_OPERATEID;
    LSB(ambe_input.DtmfCtrl) = 0xFF;
    LSB(ambe_input.Ctrl2) = AMBE_CTRL2_ENCODE;
    VoicePktPtr = (char *)SilenceFrame;
    if ((AMBEstate == AMBEoff) || (AMBEstate == AMBEpowerdown))
      ambe_start();
    else if (AMBEstate > AMBEbooting) {
      ambe_start_transmit();			// AMBE aufwecken
      ambe_idle_count = 0;
    }
  } // fi
}


void ambe_decode(void) {
  if ((AMBEstate != AMBEdecoding) && (AMBEstate != AMBEnoinit)) {
    ambe_input.Ctrl1 = AMBE_OPERATEID|AMBE_CTRL1_CNI_MASK;	// CNI on
    LSB(ambe_input.DtmfCtrl) = 0xFF;
    LSB(ambe_input.Ctrl2) = 0;	// Decoding, Set both rates
    if ((AMBEstate == AMBEoff) || (AMBEstate == AMBEpowerdown))
      ambe_start();
    else
      ambe_idle_count = 0;
  } // fi allowed states
}


void ambe_powerdown(void) {	// Kontrolliertes "runterfahren" bei Op-Mode-Wechsel
  // Test:
  //ambe_stop();
  if (AMBEstate > AMBEpowerdown) {
    AMBEstate = AMBEpowerdown;
    ambe_timer_stop();
    ambe_input.Ctrl1      = AMBE_PWRDOWNID;
    LSB(ambe_input.Ctrl2) = AMBE_CTRL2_EE_MASK;	// Encoder-Enable
    ambe_start_transmit();
    INTC_register_interrupt(ambe_timer_TurnOff, AMBE_TIMER_IRQ, AMBE_TIMER_PRIO);
    ambe_timer_setms(AMBE_FRAMETIMEOUT);
    ambe_timer_start();
    eic_disableint(AMBE_EPR_INT);
  } // fi
}


void ambe_getvoice(unsigned char *dest) {
  memcpy(dest, VoicePktPtr, AMBE_USEDDATA);	// Encoded or Const Data copy
  ambe_idle_count = 0;
}


void ambe_putvoice(unsigned char *voicedat) {
  if (AVR32_SSC.sr&AVR32_SSC_SR_TXEN_MASK) {
    // Wenn's noch nicht zu spät ist - überschreiben:
    if (AVR32_PDCA.channel[AMBE_CHANNEL1].tcr > (AMBE_FRAMESIZE/2-2))
      memcpy(ambe_input.Data, voicedat, AMBE_USEDDATA);
  } else {	// Transmitter Idle (default)
    memcpy(ambe_input.Data, voicedat, AMBE_USEDDATA);
    if (ambe_rxframe_count>0) {			// AMBE arbeitet...
      ambe_input.Ctrl1 = AMBE_OPERATEID;	// CNI, LFI off
      ambe_start_transmit();
      ambe_restart_timer();		// reset Timeout für LFI/CNI
      ambe_idle_count = 0;		// neue Daten - Reset Timeout
    } // fi nur wenn AMBE läuft
  }
}



__inline tambestate ambe_getstate(void) {
  return AMBEstate;
}

__inline tambestate ambe_getnewstate(void) {
  if (AMBEstate==prevAMBEstate)
    return AMBEnoinit;
  else {
    prevAMBEstate = AMBEstate;
    return AMBEstate;
  }
}


__inline int ambe_packetcount(void) {
  return ambe_rxframe_count;
}


unsigned short ambe_getber(void) {
  unsigned short ber = BitErrorRate;
  BitErrorRate |= 0x8000;	// LastError, must be updated with next voice pkt
  return ber;
}


__inline void ambe_set_volume(unsigned char volume) {
  float vol = -(248-volume) / 160.0;	// -15.5db (0.5dB Steps)
  vol = pow10f(vol)*AMBE_MAXVOL;

  MSB(ambe_input.Ctrl2) = round(vol); //ume;

}


void ambe_set_dtmf(unsigned char tone, int len_frames) {
  if (len_frames==0)
    ambe_dtmf_count = 0xffffffff;
  else
    ambe_dtmf_count = len_frames;
  LSB(ambe_input.DtmfCtrl) = tone;
  LSB(ambe_input.Ctrl2) = AMBE_CTRL2_RIS_NONE;
  if (AMBEstate==AMBEencoding) {
    ambe_start_transmit();
  }
}


void ambe_stop_dtmf(void) {
  ambe_dtmf_count = 1;
}


void ambe_set_timeout(unsigned int time2shutdown) {
  if (time2shutdown > 0)
    ambe_powerdown_to = time2shutdown;
}

//! @}



/*! \name AMBE Initialization
 */
//! @{

void ambe_init(void) {
  AMBEstate = AMBEnoinit;
  cpld_config(CPLDdisabled);
  cpld_start();

  // *** Synchronous Serial Controller init ***
  AVR32_SSC.cr   = 0x00008000;		// Do a software reset
  AVR32_SSC.cmr  = 1;
  AVR32_SSC.tcmr = SSC_xCMR_AMBEPAS;	// startdelay 1 clk, 20 Bits Period (9orig)
  AVR32_SSC.tfmr = SSC_xFMR_AMBEPAS;	// 16 Bits + POS PULSE
  AVR32_SSC.rcmr = SSC_xCMR_AMBEPAS;
  AVR32_SSC.rfmr = SSC_xFMR_AMBEPAS;	// RF used as Output

  INTC_register_interrupt(&ambe_ssc_int, AVR32_SSC_IRQ, AMBE_TRXSTART_PRIO);

  // *** Init PDAC Receiver for SSC ***
  AVR32_PDCA.channel[AMBE_CHANNEL0].idr = 0x0FFF;
  AVR32_PDCA.channel[AMBE_CHANNEL0].cr  = AVR32_PDCA_ECLR_MASK|AVR32_PDCA_TDIS_MASK;
  AVR32_PDCA.channel[AMBE_CHANNEL0].mar = (U32)&ambe_output;
  AVR32_PDCA.channel[AMBE_CHANNEL0].tcr = AMBE_FRAMESIZE;
  AVR32_PDCA.channel[AMBE_CHANNEL0].tcrr= 0;
  AVR32_PDCA.channel[AMBE_CHANNEL0].mr  = 0x01 << AVR32_PDCA_SIZE_OFFSET; // Half-Word
  AVR32_PDCA.channel[AMBE_CHANNEL0].psr = AVR32_PDCA_PID_SSC_RX;

  AVR32_PDCA.channel[AMBE_CHANNEL1].idr = 0x0FFF;
  AVR32_PDCA.channel[AMBE_CHANNEL1].cr  = AVR32_PDCA_ECLR_MASK|AVR32_PDCA_TDIS_MASK;
  AVR32_PDCA.channel[AMBE_CHANNEL1].mar = (U32)&ambe_input;
  AVR32_PDCA.channel[AMBE_CHANNEL1].tcr = AMBE_FRAMESIZE;
  AVR32_PDCA.channel[AMBE_CHANNEL1].tcrr= 0;
  AVR32_PDCA.channel[AMBE_CHANNEL1].mr  = 0x01 << AVR32_PDCA_SIZE_OFFSET; // Half-Word
  AVR32_PDCA.channel[AMBE_CHANNEL1].psr = AVR32_PDCA_PID_SSC_TX;

  INTC_register_interrupt(&ambe_rxfinish_int, AVR32_PDCA_IRQ_0+AMBE_CHANNEL0, AMBE_TRXEND_PRIO);
  INTC_register_interrupt(&ambe_txfinish_int, AVR32_PDCA_IRQ_0+AMBE_CHANNEL1, AMBE_TRXEND_PRIO);

  // *** Timer: Periodisch, Up bis RC-Compare, AutoTrg ***
  AVR32_TC.channel[AMBE_TIMER_CH].cmr =  AVR32_TC_NONE << AVR32_TC_BSWTRG_OFFSET |
    AVR32_TC_NONE << AVR32_TC_BEEVT_OFFSET |
    AVR32_TC_NONE << AVR32_TC_BCPC_OFFSET |
    AVR32_TC_NONE << AVR32_TC_BCPB_OFFSET |
    AVR32_TC_NONE << AVR32_TC_ASWTRG_OFFSET |
    AVR32_TC_NONE << AVR32_TC_AEEVT_OFFSET |
    AVR32_TC_NONE << AVR32_TC_ACPC_OFFSET |
    AVR32_TC_NONE << AVR32_TC_ACPA_OFFSET |
    1 << AVR32_TC_WAVE_OFFSET |
    AVR32_TC_WAVSEL_UP_AUTO << AVR32_TC_WAVSEL_OFFSET |
    FALSE << AVR32_TC_ENETRG_OFFSET |
    AVR32_TC_EEVT_TIOB_INPUT << AVR32_TC_EEVT_OFFSET |
    AVR32_TC_EEVTEDG_NO_EDGE << AVR32_TC_EEVTEDG_OFFSET |
    FALSE << AVR32_TC_CPCDIS_OFFSET |
    FALSE << AVR32_TC_CPCSTOP_OFFSET |
    AVR32_TC_BURST_NOT_GATED << AVR32_TC_BURST_OFFSET |
    0 << AVR32_TC_CLKI_OFFSET |
    AVR32_TC_TCCLKS_TIMER_CLOCK5 << AVR32_TC_TCCLKS_OFFSET;	// MASTERCLK/128

  // *** EPR - Packet-Ready-Int (one-shoot-trigger rx) ***
  eic_clrline(AMBE_EPR_INT);
  eic_enableint(AMBE_EPR_INT, &ambe_epr_int, AMBE_EPR_PRIO, EICm_FALLING_EDGE);
  cpld_config(CPLDcfg_ambepas);		// AMBE2020
  AMBEstate = AMBEoff;
}


void ambe_setup(void) {
  AMBEstate = AMBEnoinit;
  ambe_powerdown_to = 0xFFFFFFF0;	// Disable Power-Down TO (huge Timeout)
  //ambe_bitsleftfkt  = NULL;
  ambe_input.Header  = AMBE_HEADER;
  ambe_input.Rate[0] = AMBE_RATE0;
  ambe_input.Rate[1] = AMBE_RATE1;
  ambe_input.Rate[2] = AMBE_RATE2;
  ambe_input.Rate[3] = AMBE_RATE3;
  ambe_input.Rate[4] = AMBE_RATE4;
  ambe_input.Ctrl1   = (0x00<<AMBE_CTRL1_ID_OFFSET)|AMBE_CTRL1_CNI_MASK;
  ambe_input.Unused[0] = 0;
  ambe_input.Unused[1] = 0;
  ambe_input.Unused[2] = 0;
  ambe_input.DtmfCtrl = 0xFDFF;	// -3dB DTMFVol, Inactive
  ambe_input.Ctrl2 = (0x80<<AMBE_CTRL2_VOL_OFFSET)|AMBE_CTRL2_ENCODE|AMBE_CTRL2_RIS_BOTH;
  memset(&ambe_input.Data, 0, sizeof(ambe_input.Data));	// Clear Data
  VoicePktPtr = (char *)SilenceFrame;
}

//! @}

#endif	//not AMBEACT
