/*
 * dac_func.c
 *
 * Basic functions of the AD5620 connected via SPI interface.
 *
 *  Created on: 11.03.2009
 *      Author: Jan Alte, DO1FJN
 *
 *
 * This file is part of the DV-modem firmware (DVfirmware).
 * For general infomation about DVfirmware see "dv-main.c".
 *
 * DVfirmware is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * The DVfirmware is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this package. If not, see <http://www.gnu.org/licenses/>.
 *
 */


#include "dac_func.h"
#include "hw_defs.h"
#include "compiler.h"


#define spi_csr	((0<<AVR32_SPI_DLYBCT_OFFSET)|(1<<AVR32_SPI_DLYBS_OFFSET)|	\
    (((MASTERCLOCK+DAC_CLOCK/2)/DAC_CLOCK)<<AVR32_SPI_SCBR_OFFSET)|			\
    (8<<AVR32_SPI_BITS_OFFSET)|AVR32_SPI_CPOL_MASK|AVR32_SPI_NCPHA_MASK)


void dac_init(void) {
  // ToDo: pm_enable-SPI()...
  AVR32_SPI.cr = AVR32_SPI_CR_SWRST_MASK;	// Reset
  // Master Mode, no Peripheral Selected
  AVR32_SPI.mr = (1<<AVR32_SPI_MR_DLYBCS_OFFSET)|(0x0f<<AVR32_SPI_MR_PCS_OFFSET)|AVR32_SPI_MSTR_MASK;
#if (SPI_CS==0)
  AVR32_SPI.csr0 = spi_csr;
#elif (SPI_CS==1)
  AVR32_SPI.csr1 = spi_csr;
#elif (SPI_CS==2)
  AVR32_SPI.csr2 = spi_csr;
#elif (SPI_CS==3)
  AVR32_SPI.csr3 = spi_csr;
#endif
  AVR32_SPI.mr &= ~(1 << (AVR32_SPI_MR_PCS_OFFSET + SPI_CS));
  AVR32_SPI.cr = AVR32_SPI_CR_SPIEN_MASK;
}


void dac_exit(void) {
  dac_transmit(0x8000);	// Powerdown 100k-GND
  dac_waitidle();
  AVR32_SPI.cr = AVR32_SPI_CR_SPIDIS_MASK;
  // ToDo:
  // PM-Disable-SPI()... to save some µA power
}


__inline void dac_transmit(unsigned short int data) {
  AVR32_SPI.tdr = data;
}


void dac_waitidle(void) {
  while((AVR32_SPI.sr&AVR32_SPI_SR_TXEMPTY_MASK)==0);
}

