/*
 * hw_def09.h
 *
 * Definition of AVR32 peripherals, GPIO pin functions and Int-Usages.
 * This file is for Hardware 0.9PT only.
 *
 *  Created on: 08.03.2009
 *      Author: Jan Alte, DO1FJN
 *
 *
 * This file is part of the DV-modem firmware (DVfirmware).
 * For general infomation about DVfirmware see "dv-main.c".
 *
 * DVfirmware is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * The DVfirmware is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this package. If not, see <http://www.gnu.org/licenses/>.
 *
 */


#ifndef HW_DEF09_H_
#define HW_DEF09_H_

#define MASTERCLOCK		57344000L
#define OSC0STARTUPVALUE	0

#define AMBEACT			// AMBE can operate ACTIVE only (hard wired)

// DAC Standard Values for GMSK / D-Star
#define DAC_MIDDLE		0		// Signed Offset to real Bit-Middle
#define DAC_STDHUB		6500

// use SSC TF-Workaround?
#define SSC_TFDIRECTION		1		// 1 = Use TF as Input (connected with RF)

#define SSC_TCMR_INI		0x00000422	// Start=3 (High TF Level), with CKI, TK Pin is Src
#define SSC_TFMR_INI		0x0000008F	// 16 Bits, MSB first

#define SSC_RCMR_CODEC		0x00000402	// RK-Input, Start falling RF
#define SSC_RFMR_CODEC		0x0000008F	// RF: Input, 16Bit Len, MSB First
#define SSC_RCMR_AMBEACT	0x00000402	// RK-Input, Start falling RF
#define SSC_RFMR_AMBEACT	0x0000008F	// RF: Input, 16Bit Len, MSB First


// GPIO - FUNCTION PIN Definition:
#define MODEN_PIN		AVR32_PIN_PA08
#define WATCHDOG_PIN		AVR32_PIN_PA03
#define AMBE_RESET_PIN		AVR32_PIN_PA12	// OpenDrain, hold low min. 50µs
#define RUNCFG_PIN		AVR32_PIN_PA17

#define USBID_PIN		AVR32_PIN_PA26
#define USBVBOF_PIN		AVR32_PIN_PA27

#define HFIN_PIN		AVR32_ADC_AD_7_PIN
#define RSSI_PIN		AVR32_ADC_AD_6_PIN

#define BFRXD_PIN		AVR32_USART0_RXD_0_0_PIN
#define BFTXD_PIN		AVR32_USART0_TXD_0_0_PIN
#define BFREQ_PIN		AVR32_PIN_PA22

// GMSK-Debug-Pins
#define DBG_PIN0		AVR32_PIN_PA20
#define DBG_PIN1		AVR32_PIN_PA21

#define UNUSED_IN0		((1<<AVR32_PIN_PA04)|(1<<AVR32_PIN_PA05)|(1<<AVR32_PIN_PA06)| \
				(1<<AVR32_PIN_PA07)|(1<<AVR32_PIN_PA11)|(1<<AVR32_PIN_PA24)| \
				(1<<AVR32_PIN_PA29))


#define CPLDMODE_PIN		(AVR32_PIN_PB00-32)
#define CPLDTFDIR_PIN		(AVR32_PIN_PB01-32)

// RS232 connected on Port B
#define RS232RXD_PIN		(AVR32_USART1_RXD_0_1_PIN-32)
#define RS232TXD_PIN		(AVR32_USART1_TXD_0_1_PIN-32)
// RS232: Function "2" RxD/TxD & "0" CTS/RTS
#define RS232CTS_PIN		(AVR32_USART1_CTS_0_0_PIN-32)
#define RS232RTS_PIN		(AVR32_USART1_RTS_0_0_PIN-32)



// *** Definition of Port A ***
#define GPIO0_PULLUP		(UNUSED_IN0)

#define GPIO0_ODER		(1<<DBG_PIN0)|(1<<DBG_PIN1)| \
				(1<<MODEN_PIN)|(1<<WATCHDOG_PIN)| \
				(1<<RUNCFG_PIN)|(1<<USBVBOF_PIN)
#define GPIO0_OVRINIT		(1<<RUNCFG_PIN)| \
                                (1<<MODEN_PIN)

#define GPIO0_PMR0		0
#define GPIO0_PMR1		0	//(1<<BFRXD_PIN)|(1<<BFTXD_PIN)
#define GPIO0_DISABLE_MASK	(1<<BFRXD_PIN)|(1<<BFTXD_PIN)|	\
	                        (1<<AVR32_TWI_SDA_0_0_PIN)|(1<<AVR32_TWI_SCL_0_0_PIN)|	\
	                        (1<<AVR32_SPI_MOSI_0_0_PIN)|(1<<AVR32_SPI_SCK_0_0_PIN)|	\
	                        (1<<AVR32_SPI_NPCS_0_0_PIN)|	\
	                        (1<<HFIN_PIN)|(1<<RSSI_PIN)

// *** Definition of Port B ***

#define GPIO1_PULLUP		0
#define GPIO1_ODER		(1<<CPLDMODE_PIN)|(1<<CPLDTFDIR_PIN)
#define GPIO1_OVRINIT		0
#define GPIO1_PMR0		0
#define GPIO1_PMR1		(1<<RS232RXD_PIN)|(1<<RS232TXD_PIN)

#define GPIO1_DISABLE_MASK	(1<<(RS232RXD_PIN&0x1F))|(1<<(RS232TXD_PIN&0x1F))|	\
                                (1<<(RS232CTS_PIN&0x1F))|(1<<(RS232RTS_PIN&0x1F))|	\
                                (1<<(AVR32_SSC_RX_CLOCK_0_PIN&0x1F))| \
                                (1<<(AVR32_SSC_RX_DATA_0_PIN&0x1F))| \
                                (1<<(AVR32_SSC_RX_FRAME_SYNC_0_PIN&0x1F))| \
                                (1<<(AVR32_SSC_TX_CLOCK_0_PIN&0x1F))| \
                                (1<<(AVR32_SSC_TX_DATA_0_PIN&0x1F))| \
                                (1<<(AVR32_SSC_TX_FRAME_SYNC_0_PIN&0x1F))


// End GPIO Section

#define PBAMASK_DEFAULT		0x1FFF
#define PBAMASK_NEEDED		0x18FF	// Disable Clock for USART1+2, PWM


// TWI (I²C) Definition:
#define POTI_ADR		0x2F

#define BFIO			AVR32_USART0
#define BFIRQ			AVR32_USART0_IRQ
#define BFPIDRX			AVR32_PDCA_PID_USART0_RX
#define BFPIDTX			AVR32_PDCA_PID_USART0_TX


#endif // HW_DEFS_H_
