/*
 * hw_defs.h
 *
 * Definition of AVR32 peripherals, GPIO pin functions and Int-Usages.
 * This file only for HW1.0 and later. For HW0.9PT a redirection to "hw-def09.h" is
 * included.
 *
 *  Created on: 08.03.2009
 *      Author: Jan Alte, DO1FJN
 *
 *
 * This file is part of the DV-modem firmware (DVfirmware).
 * For general infomation about DVfirmware see "dv-main.c".
 *
 * DVfirmware is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * The DVfirmware is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this package. If not, see <http://www.gnu.org/licenses/>.
 *
 *
 *  Report:
 *  2010-01-16	Ausgemistet.
 *  2010-02-14  RS232 (USART 1) hinzu
 */


#ifndef HW_DEFS_H_
#define HW_DEFS_H_


// PDCA Channels for AMBE:
#define AMBE_CHANNEL0		0
#define AMBE_CHANNEL1		1
// PDCA Channel for Serial BF-Bus:
#define BF_CHANNEL		2	// one Channel for RX and TX
// PDCA Channels for RS232 Interface:
#define RS232_RXCH		3
#define RS232_TXCH		4

// AMBE - Priorities
#define AMBE_EPR_PRIO		AVR32_INTC_INT1	// >= als TXRDY, damit vor TRXEND
#define AMBE_TRXSTART_PRIO	AVR32_INTC_INT3
#define AMBE_TRXEND_PRIO	AVR32_INTC_INT1

// Externe Interrupts EIC & NMI:
#define AMBE_EPR_INT		AVR32_EIC_INT3
#define AMBE_EPR_PIN		AVR32_EIC_EXTINT_3_PIN
#define AMBE_EPR_FUNCTION	AVR32_EIC_EXTINT_3_FUNCTION

// GMSK Timer + Ints
#define GMSK_TIMER_CH		0
#define GMSK_TIMER_IRQ		AVR32_TC_IRQ0
#define GMSK_MOD_INTPRIO	AVR32_INTC_INT2
#define GMSK_DEMOD_INTPRIO	AVR32_INTC_INT2

#define AMBE_TIMER_CH		1
#define AMBE_TIMER_IRQ		AVR32_TC_IRQ1
#define AMBE_TIMER_PRIO		AVR32_INTC_INT0

#define IDLE_TIMER_CH		1	// gleich AMBE-Timer, da nie zusammen ben.
#define	IDLE_TIMER_IRQ		AVR32_TC_IRQ1


#ifdef HW09PT

#include "hw_def09.h"

#else

#define MASTERCLOCK		60000000L

#define OSC0STARTUPVALUE	4

// use SSC TF-Workarround?
#define SSC_TFDIRECTION		1		// 1 = Use TF as Input (connectet with RF)

//#define SSC_RCMR_INI		0x00000402	// RK-Input, Start falling RF
//#define SSC_RFMR_INI		0x0000008F	// RF-Input, 16Bit Len, MSB First
#define SSC_TCMR_INI		0x00000422	// Start=3 (High TF Level), with CKI, TK Pin is Src
#define SSC_TFMR_INI		0x0000008F	// 16 Bits, MSB first

#define SSC_RCMR_CODEC		0x00000402	// RK-Input, Start falling RF
#define SSC_RFMR_CODEC		0x0000008F	// RF: Input, 16Bit Len, MSB First
#define SSC_RCMR_AMBEACT	0x00000402	// RK-Input, Start falling RF
#define SSC_RFMR_AMBEACT	0x0000008F	// RF: Input, 16Bit Len, MSB First
// AMBE passive framed mode:
// use a min. gap of 25 SSC-Clock-Cylces to prevent timing errors
#define SSC_xCMR_AMBEPAS	0x13000402	// RK-Input, Start falling RF, Period 40Clks
#define SSC_xFMR_AMBEPAS	0x0020008F	// RF: Pos.Pulse, 16Bit Len, MSB First, 24 Data

// DAC Standard Values for GMSK / D-Star
#define DAC_MIDDLE		0		// Signed Offset to real Bit-Middle
#define DAC_STDHUB		6500

// GPIO Section (Defines)

// Output-GPIO-Pins (PortA)
#ifdef HW10
#define PTT_PIN			AVR32_PIN_PA05
#define GREEN_LED		AVR32_PIN_PA06
#define RED_LED			AVR32_PIN_PA07
#else
#define GREEN_LED		AVR32_PIN_PA05
#define RED_LED			AVR32_PIN_PA06
#define PTT_PIN			AVR32_PIN_PA07
#endif

// Debug-Pins
#define DBG_PIN0		RED_LED
#define DBG_PIN1		GREEN_LED

#define MODEN_PIN		AVR32_PIN_PA08
#define WATCHDOG_PIN		AVR32_PIN_PA11

#define CPLDRKTKDIR_PIN		AVR32_PIN_PA17
#define CPLDTFDIR_PIN		AVR32_PIN_PA22
#define RUNCFG_PIN		AVR32_PIN_PA24

#define USBID_PIN		AVR32_PIN_PA26
#define USBVBOF_PIN		AVR32_PIN_PA27

#define HFIN_PIN		AVR32_ADC_AD_7_PIN
#define RSSI_PIN		AVR32_ADC_AD_6_PIN

#define UNUSED_IN0		(1<<AVR32_PIN_PA03)|(1<<AVR32_PIN_PA04)|(1<<AVR32_PIN_PA25)| \
				(1<<AVR32_PIN_PA28)|(1<<AVR32_PIN_PA29)

// Output-GPIO-Pins (PortB)
#define CPLDCFG0_PIN		(AVR32_PIN_PB01-32)
#define CPLDCFG1_PIN		(AVR32_PIN_PB00-32)


#define BFRXD_PIN		AVR32_USART2_RXD_0_1_PIN
#define BFTXD_PIN		AVR32_USART2_TXD_0_1_PIN
// BF-Bus-Pins: Function "2"

// RS232 connected on Port B
#define RS232RXD_PIN		(AVR32_USART1_RXD_0_1_PIN-32)
#define RS232TXD_PIN		(AVR32_USART1_TXD_0_1_PIN-32)
// RS232: Function "2" RxD/TxD & "0" CTS/RTS
#define RS232CTS_PIN		(AVR32_USART1_CTS_0_0_PIN-32)
#define RS232RTS_PIN		(AVR32_USART1_RTS_0_0_PIN-32)
#define RS232OFF_PIN		AVR32_PIN_PA12


// *** Definition of Port A ***
#define GPIO0_PULLUP		(1<<BFTXD_PIN)|(UNUSED_IN0)

#define GPIO0_ODER		(1<<PTT_PIN)|(1<<RED_LED)|(1<<GREEN_LED)|(1<<RS232OFF_PIN)| \
				(1<<MODEN_PIN)|(1<<WATCHDOG_PIN)|(1<<CPLDRKTKDIR_PIN)| \
				(1<<CPLDTFDIR_PIN)|(1<<RUNCFG_PIN)|(1<<USBVBOF_PIN)
#define GPIO0_OVRINIT		(1<<PTT_PIN)|(1<<GREEN_LED)|(0<<RS232OFF_PIN)|(1<<RUNCFG_PIN)| \
                                (1<<MODEN_PIN)
// red LED light up (invert logic)

#define GPIO0_PMR0		0
#define GPIO0_PMR1		(1<<BFRXD_PIN)|(1<<BFTXD_PIN)
#define GPIO0_DISABLE_MASK	(1<<BFRXD_PIN)|(1<<BFTXD_PIN)|	\
	                        (1<<AVR32_TWI_SDA_0_0_PIN)|(1<<AVR32_TWI_SCL_0_0_PIN)|	\
	                        (1<<AVR32_SPI_MOSI_0_0_PIN)|(1<<AVR32_SPI_SCK_0_0_PIN)|	\
	                        (1<<AVR32_SPI_NPCS_0_0_PIN)|	\
	                        (1<<HFIN_PIN)|(1<<RSSI_PIN)

// *** Definition of Port B ***

#define GPIO1_PULLUP		0
#define GPIO1_ODER		(1<<CPLDCFG0_PIN)|(1<<CPLDCFG1_PIN)
#define GPIO1_OVRINIT		0
#define GPIO1_PMR0		0
#define GPIO1_PMR1		(1<<RS232RXD_PIN)|(1<<RS232TXD_PIN)

#define GPIO1_DISABLE_MASK	(1<<(RS232RXD_PIN&0x1F))|(1<<(RS232TXD_PIN&0x1F))|	\
                                (1<<(RS232CTS_PIN&0x1F))|(1<<(RS232RTS_PIN&0x1F))|	\
                                (1<<(AVR32_SSC_RX_CLOCK_0_PIN&0x1F))| \
                                (1<<(AVR32_SSC_RX_DATA_0_PIN&0x1F))| \
                                (1<<(AVR32_SSC_RX_FRAME_SYNC_0_PIN&0x1F))| \
                                (1<<(AVR32_SSC_TX_CLOCK_0_PIN&0x1F))| \
                                (1<<(AVR32_SSC_TX_DATA_0_PIN&0x1F))| \
                                (1<<(AVR32_SSC_TX_FRAME_SYNC_0_PIN&0x1F))


// End GPIO Section

#define PBAMASK_DEFAULT		0x1FFF
#define PBAMASK_NEEDED		0x1B7F	// Disable Clock for USART0, PWM


// Peripheral Definitions:
#define BFIO			AVR32_USART2
#define BFIRQ			AVR32_USART2_IRQ
#define BFPIDRX			AVR32_PDCA_PID_USART2_RX
#define BFPIDTX			AVR32_PDCA_PID_USART2_TX

#define RS232			AVR32_USART1
#define RS232IRQ		AVR32_USART1_IRQ
#define RS232PIDRX		AVR32_PDCA_PID_USART1_RX
#define RS232PIDTX		AVR32_PDCA_PID_USART1_TX


#endif // esle DEFAULT Hardware


#endif // HW_DEFS_H_
